@@ -1,47 +1,49 @@
 read.ctv <- function(file)
 {
   ## XML package is only needed here (i.e., only server-side)
-  if(!("package:XML" %in% search())) stopifnot(require(XML))
+  if(!("package:XML" %in% search())) stopifnot(require("XML"))
 
   ## read raw XML
   x <- xmlTreeParse(file)
   if(xmlSize(x$doc$children) > 1) warning("ctv should contain only one view")
-  x <- xmlChildren(x$doc$children[[1]])
-  
+  x <- xmlChildren(x$doc$children$CRANTaskView)
+
   ## valid task view?
-  ctvNames <- c("name", "topic", "maintainer", "info", "packagelist", "links")
+  ctvNames <- c("name", "topic", "maintainer", "version", "info", "packagelist", "links")
   missingChildren <- !(ctvNames %in% names(x))
-  if(any(missingChildren)) stop(paste("The following ctv nodes are missing:", ctvNames[missingChildren]))
-  
+  if(any(missingChildren))
+    stop("The following ctv nodes are missing: ",
+      paste(ctvNames[missingChildren], collapse=", "))
+
   ## convenience function for transforming
   ## XMLNodes into a character vector
   xmlPaste <- function(x, indent = "",
                        viewprefix = "",
-                       packageURL = "../Descriptions/",
+                       packageURL = "../packages/",
                        viewURL = "")
   {
     ## get tag name
     name <- xmlName(x, full = TRUE)
-  
+
     ## if final node, return text
     if(name == "text")
       return(paste(indent, xmlValue(x), sep = ""))
 
     ## if pkg or view, return link
-    if(name == "pkg") 
-      return(paste("<a href=\"", packageURL, xmlValue(x), ".html\">", xmlValue(x), "</a>", sep = ""))
-    if(name == "view") 
-      return(paste(viewprefix, "<a href=\"", viewURL, xmlValue(x), ".html\">", xmlValue(x), "</a>", sep = ""))    
+    if(name == "pkg")
+      return(paste("<a href=\"", packageURL, xmlValue(x), "/index.html\">", xmlValue(x), "</a>", sep = ""))
+    if(name == "view")
+      return(paste(viewprefix, "<a href=\"", viewURL, xmlValue(x), ".html\">", xmlValue(x), "</a>", sep = ""))
     if(name == "info")
       name <- "p"
-  
+
     ## get attributes
     tmp <- if(!is.null(xmlAttrs(x)))
       paste(names(xmlAttrs(x)), paste("\"", xmlAttrs(x), "\"", sep = ""), sep = "=", collapse = " ")
       else ""
 
     ## start tag
-    rval <- paste(indent, "<", name, ifelse(tmp != "", " ", ""), tmp, ">", sep = "")  
+    rval <- paste(indent, "<", name, ifelse(tmp != "", " ", ""), tmp, ">", sep = "")
     ## content
     subIndent <- paste(indent, "  ", sep = "")
     for(i in xmlChildren(x))
@@ -63,6 +65,8 @@ read.ctv <- function(file)
   name <- function(x) xmlValue(x$name)
   topic <- function(x) xmlValue(x$topic)
   maintainer <- function(x) xmlValue(x$maintainer)
+  email <- function(x) as.vector(xmlAttrs(x$maintainer)["email"])
+  ctvversion <- function(x) xmlValue(x$version)
   info <- function(x) newlineSub(xmlPaste(x$info, indent = "    "))
   package1 <- function(x) {
     rval <- xmlAttrs(x)["priority"]
@@ -73,16 +77,17 @@ read.ctv <- function(file)
     rval <- t(sapply(xmlChildren(x$packagelist), package1))
     colnames(rval) <- NULL
     rownames(rval) <- NULL
-    rval <- data.frame(name = I(rval[,1]), core = rval[,2] == "core")    
+    rval <- data.frame(name = I(rval[,1]), core = rval[,2] == "core")
     rval[order(tolower(rval[,1])), ]
   }
   links <- function(x) as.vector(xmlSApply(x$links, function(z) xmlPaste(z, viewprefix = "CRAN Task View: ")))
 
-
   ## collect nodes and return
   rval <- list(name = name(x),
                topic = topic(x),
 	       maintainer = maintainer(x),
+	       email = email(x),
+	       version = ctvversion(x),
 	       info = info(x),
 	       packagelist = packagelist(x),
 	       links = links(x))
@@ -92,18 +97,18 @@ read.ctv <- function(file)
 
 ctv2html <- function(x,
                      file = NULL,
-                     css = "../../../R.css",
-		     packageURL = "../Descriptions/",
+                     css = "../../R.css",
+		     packageURL = "../packages/",
 		     reposname = "CRAN")
 {
   if(is.character(x)) x <- read.ctv(x)
   if(is.null(file)) file <- paste(x$name, ".html", sep = "")
-  
+
   ## auxiliary functions
   ampersSub <- function(x) gsub("&", "&amp;", x)
   zpaste <- function(..., sep = "", collapse = NULL) paste(..., sep = sep, collapse = collapse)
 
-  ## create HTML  
+  ## create HTML
   ## header
   htm1 <- c("<html>",
             "<head>",
@@ -113,18 +118,22 @@ ctv2html <- function(x,
 	    "",
 	    "<body>",
      zpaste("  <h2>", reposname, " Task View: ", ampersSub(x$topic), "</h2>"),
-     zpaste("  <h3>Maintainer: ", ampersSub(x$maintainer), "</h3>"))
+            "  <table>",
+     zpaste("    <tr><td valign=\"top\"><b>Maintainer:</b></td><td>", ampersSub(x$maintainer), "</td></tr>"),
+     if(!is.null(x$email)) zpaste("    <tr><td valign=\"top\"><b>Contact:</b></td><td>", gsub("@", " at ", x$email), "</td></tr>"),
+     zpaste("    <tr><td valign=\"top\"><b>Version:</b></td><td>", ampersSub(x$version), "</td></tr>"),
+            "  </table>")
 
-  ## info section	    
+  ## info section
   htm2 <- ampersSub(x$info)
 
   ## package list
   pkg2html <- function(a, b)
-    zpaste("    <li><a href=\"", packageURL, a, ".html\">", a, "</a>",
+    zpaste("    <li><a href=\"", packageURL, a, "/index.html\">", a, "</a>",
            if(b) " (core)" else "", "</li>")
   htm3 <- c(zpaste("  <h3>", reposname, " packages:</h3>"),
             "  <ul>",
-	    sapply(1:NROW(x$packagelist), function(i) pkg2html(x$packagelist[i,1], x$packagelist[i,2])), 
+	    sapply(1:NROW(x$packagelist), function(i) pkg2html(x$packagelist[i,1], x$packagelist[i,2])),
 	    "  </ul>")
 
   ## further links
@@ -133,7 +142,7 @@ ctv2html <- function(x,
             sapply(x$links, function(x) paste("    <li>", ampersSub(x), "</li>", sep = "")),
 	    "  </ul>")
 
-  ## collect code chunks	    
+  ## collect code chunks
   htm <- c(htm1, "", htm2, "", htm3, "", htm4, "", "</body>", "</html>")
 
   ## write and return results
@@ -141,41 +150,40 @@ ctv2html <- function(x,
   invisible(htm)
 }
 
-updateViews <- function(repos = ".",
-			css = "../../../R.css",
-			reposname = "CRAN",
-			...)
+repos_update_views <- function(repos = ".",
+			       css = "../../R.css",
+		   	       reposname = "CRAN",
+			       ...)
 {
   ## These could easily be changed, but are currently not
   ## exported arguments
   viewsrds <- "Views.rds"
   index <- "index.html"
 
-  ## list all .ctv files
-  path <- file.path(repos, "src", "contrib", "Views")
-  files <- dir(path, pattern = "\\.ctv$")
-  if(length(files) < 1) stop(paste("no .ctv files found at path", path))
-  
-  ## compute correct installdirs
-  contribdir <- file.path(repos, "src", "contrib")
-  viewdir <- file.path(contribdir, "Views")
-  viewsrds <- file.path(contribdir, viewsrds)
+  ## compute correct installation dirs
+  viewdir <- file.path(repos, "web", "views")
   index <- file.path(viewdir, index)
-  
+  contribdir <- file.path(repos, "src", "contrib")
+  viewsrds <- file.path(contribdir, viewsrds)
+
+  ## available views
+  files <- dir(viewdir, pattern = "\\.ctv$")
+  if(length(files) < 1) stop(paste("no .ctv files found at path", viewdir))
+
   ## available packages in repos
   pkgs <- as.vector(read.dcf(file.path(contribdir, "PACKAGES"), fields = "Package"))
-  
+
   ## setup object which will be stored in Views.rds
   rval <- list()
   class(rval) <- "ctvlist"
-  
+
   ## setup information for index.html
   idx <- matrix(rep("", 2 * length(files)), ncol = 2)
-  
+
   for(i in 1:length(files)) {
     ## read .ctv and store in list
-    x <- read.ctv(file.path(path, files[i]))
-    
+    x <- read.ctv(file.path(viewdir, files[i]))
+
     ## generate HTML code
     ctv2html(x, file = file.path(viewdir, paste(x$name, ".html", sep = "")),
              css = css, reposname = reposname, ...)
@@ -198,10 +206,10 @@ updateViews <- function(repos = ".",
     idx[i,] <- c(x$name, x$topic)
     rval[[i]] <- x
   }
-  
-  ## save all views 
+
+  ## save all views
   .saveRDS(rval, file = viewsrds) ## compress = TRUE currently does not work for reading from an url
-  
+
   ## generate index HTML file
   if(is.character(index)) {
     idx <- c("<html>",
@@ -221,13 +229,17 @@ updateViews <- function(repos = ".",
 	     "",
              "<p>To automatically install these views, the ctv package needs to be installed, e.g., via<br>",
 	     "   <tt>install.packages(\"ctv\")</tt><br>",
-	     "   and then the views can be installed via <tt>install.views</tt> (after loading ctv), e.g.,<br>",
-	     "   <tt>install.views(\"Econometrics\")</tt></p>",
+	     "   <tt>library(\"ctv\")</tt><br>",
+	     "   and then the views can be installed via <tt>install.views</tt> or <tt>update.views</tt>",	     
+	     "   (which first assesses which of the packages are already installed and up-to-date), e.g.,<br>",
+	     "   <tt>install.views(\"Econometrics\")</tt><br>",
+	     "   or<br>",
+	     "   <tt>update.views(\"Econometrics\")</tt></p>",
 	     "",
 	     "</body>",
 	     "</html>")
     writeLines(idx, con = index)
-  }  
-  
+  }
+
   invisible(rval)
 }
